<?php

namespace BancoTalentos\Services;

use BancoTalentos\Util\{ BtParameters, DateUtil, DoctrineEntityUtil, Messages, NovoCodigo, Repositories, Service, Util };
use BancoTalentos\Services\{ ParceiroNegocioService, PessoaHService, PessoaService };

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

use Doctrine\ORM\EntityManager;

class OperadorService extends Service {

    private const ENTITY_DEFAULT_CONFIG = [
        'SET_DEFAULT_FIELDS'    => false,
        'PERSIST'               => false,
        'FLUSH'                 => false,
        'PENDING'               => true,        
    ];

    public function __construct(EntityManager $entityManager, ParceiroNegocioService $parceiroNegocioService, PessoaService $pessoaService, PessoaHService $pessoaHService, EnderecoParcService $enderecoParcService) {
        $this->entityManager            = $entityManager;
        $this->parceiroNegocioService   = $parceiroNegocioService;
        $this->pessoaService            = $pessoaService;
        $this->pessoaHService           = $pessoaHService;
        $this->enderecoParcService      = $enderecoParcService;
    }

    public function getRepository() {
        return Repositories::OPERADOR;
    }

    /**
     *  Rotina responsavel por realizar o cadastro inicial de operadores. É utilizada por rota publica. Nao eh possivel a utilizacao de variaveis de sessao
     * 
     *  @param  array   $row
     * 
     *  @return bool
     */
    public function setOperadorAcesso(array $row) {
        $requiredFields = [
            'NMOPERADOR', 'CPF', 'DSEMAILOPER', 'PASSWORD',
            'PASSWORD_CONFIRMATION', 'DTNASC', 'IDSEXOPESSOA', 'CDESTACIVIL',
            'CDPAIS', 'NRGRAUINSTR', 'IDPOSSUIFILHO', 'CDPAIS_ENDERECO',
            'SGESTADO_ENDERECO', 'CDMUNICIPIO_ENDERECO',
        ];

        // Required fields validation
        Util::checkRequiredFields($row, $requiredFields);

        // Fields format
        $row = $this->formatFields($row);

        $this->entityManager->beginTransaction();

        try {

            /* Etapa 1: Dados de login (Parcnegocio + Operador) */
            $parcNegocio = $this->setParcNegocioDefaultOrg(
                $row['NMOPERADOR'],
                $row['CPF'],
                $row['DTNASC']
            );

            $operador = $this->setOperadorDefaultOrg(
                $row['NMOPERADOR'],
                $row['DSEMAILOPER'],
                $row['DTNASC'],
                $parcNegocio->getNrparcnegocio(),
                $row['CDPAIS_ENDERECO'],
                $row['SGESTADO_ENDERECO'],
                $row['CDMUNICIPIO_ENDERECO'],
                $row['PASSWORD'],
                $row['PASSWORD_CONFIRMATION']
            );

            /* Etapa 2: Dados Pessoais (Pessoa) */
            [$pessoa, $pessoaH] = $this->setPessoaDefaultOrg(
                $row['NMOPERADOR'],
                $parcNegocio->getNrparcnegocio(),
                $row['DTNASC'],
                $row['CPF'],
                $row['IDSEXOPESSOA'],
                $row['CDESTACIVIL'],
                $row['CDPAIS'],
                $row['SGESTADO_ENDERECO'],
                $row['CDMUNICIPIO_ENDERECO'],
                $row['NRGRAUINSTR'],
                $row['IDPOSSUIFILHO']
            );

            /* Etapa 3: Endereco (Comunicaparc) */
            $enderecoParc = $this->setEnderecoParcDefaultOrg(
                $parcNegocio->getNrparcnegocio(),
                $row['CDPAIS_ENDERECO'],
                $row['SGESTADO_ENDERECO'],
                $row['CDMUNICIPIO_ENDERECO']
            );

            Util::getDoctrineEntityUtil()->persistPending();

            $this->entityManager->flush();
            $this->entityManager->commit();

            return true;
        } catch (\Exception $e) {
            $this->entityManager->rollBack();
            throw $e;
        }
    }

    public function formatFields(array $row) {
        $row['CPF'] = Util::sanitizeCPF($row['CPF']);
        $row['NMOPERADOR'] = strtoupper(Util::removeCaracteresEspeciais($row['NMOPERADOR']));
        return $row;
    }

    public function setOperadorDefaultOrg(
        string  $nmOperador,
        string  $dsEmailOper,
        string  $dtNasc,
        string  $nrParcNegocio,
        string  $cdPais,
        string  $sgEstado,
        string  $cdMunicipio,
        string  $password,
        string  $passwordConfirmation
    ) {
        if ($password !== $passwordConfirmation) {
            throw new \Exception(Messages::OPER_PASSWORD_MISMATCH);
        }

        $newOperador = $this->set(
            [
                'CDOPERADOR'        => Util::getNextCdoperadorAvailable(),
                'NMOPERADOR'        => $nmOperador,
                'DSEMAILOPER'       => $dsEmailOper,
                'DTNASCIMENTO'      => DateUtil::getDataDeString($dtNasc, DateUtil::FORMATO_BRASILEIRO, true),
                'CDPAIS'            => $cdPais,
                'SGESTADO'          => $sgEstado,
                'CDMUNICIPIO'       => $cdMunicipio,
                'CDSENHAOPERWEB'    => md5($password),
                'NRPARCNEGOCIO'     => $nrParcNegocio,
                'DTINCLUSAO'        => DateUtil::getDataAtual(),
                'NRORG'             => Util::DEFAULT_ORG,
            ],
            self::ENTITY_DEFAULT_CONFIG
        );

        return $newOperador;
    }

    public function setParcNegocioDefaultOrg(
        string  $nmOperador, 
        string  $cpf, 
        string  $dtNasc
    ) {
        $newParcNegocio = $this->parceiroNegocioService->set(
            [
                'NRPARCNEGOCIO'         => NovoCodigo::geraCodigo(Util::DEFAULT_ORG, 'PARCNEGOCIO', 1, 12, true),
                'NMPRINCIPALPARC'       => $nmOperador,
                'NMSECUNDARIPARC'       => $nmOperador,
                'NRINSCRICAOPARC'       => $cpf,
                'DTNASCIFUNDPARC'       => DateUtil::getDataDeString($dtNasc, DateUtil::FORMATO_BRASILEIRO, true),
                'CDTIPOINSCRICAO'       => 'CPF',
                'CDTIPOPARCPRINCIPAL'   => 'PESSOA',
                'IDPESSOAFISICA'        => 'S',
                'DTINCLUSAO'            => DateUtil::getDataAtual(),
                'NRORG'                 => Util::DEFAULT_ORG,
            ],
            self::ENTITY_DEFAULT_CONFIG
        );

        return $newParcNegocio;
    }

    public function setPessoaDefaultOrg(
        string  $nmPessoa,
        string  $nrParcNegocio,
        string  $dtNasc,
        string  $cpf,
        string  $idSexoPessoa,
        string  $cdEstaCivil,
        string  $cdPais,
        string  $sgEstado,
        string  $cdMunicipio,
        string  $nrGrauInstr,
        string  $idPossuiFilho
    ) {
        $newPessoa = $this->pessoaService->set(
            [
                'NRPESSOA'      => NovoCodigo::geraCodigo(Util::DEFAULT_ORG, 'GPE_PESSOA', 1, 12, true),
                'NRPARCNEGOCIO' => $nrParcNegocio,
                'DTINCLUSAO'    => DateUtil::getDataAtual(),
                'NRORG'         => Util::DEFAULT_ORG,
            ],
            self::ENTITY_DEFAULT_CONFIG
        );

        [$primeiroNome, $segundoNome, $ultimoNome] = Util::decompoeNomeCompleto($nmPessoa);

        $newPessoaH = $this->pessoaHService->set(
            [
                'NRPESSOA'          => $newPessoa->getNrpessoa(),
                'NRPESSOAH'         => NovoCodigo::geraCodigo(Util::DEFAULT_ORG, 'GPE_PESSOAH', 1, 12, true),
                'DTMESCOMPETENC'    => DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataAtual()),
                'NMPESSOA'          => $nmPessoa,
                'DTNASCPESSOA'      => DateUtil::getDataDeString($dtNasc, DateUtil::FORMATO_BRASILEIRO, true),
                'NRCPFPESSOA'       => $cpf,
                'IDSEXOPESSOA'      => $idSexoPessoa,
                'CDESTACIVIL'       => $cdEstaCivil,
                'CDPAIS'            => $cdPais,
                'SGESTADO'          => $sgEstado,
                'CDMUNICIPIO'       => $cdMunicipio,
                'NRGRAUINSTR'       => $nrGrauInstr,
                'IDPOSSUIFILHO'     => $idPossuiFilho,
                'DSPRINOMEPESS'     => $primeiroNome,
                'DSNOMEMEIOPESS'    => $segundoNome,
                'DSULTNOMEPESS'     => $ultimoNome,
                'DTINCLUSAO'        => DateUtil::getDataAtual(),
                'NRORG'             => Util::DEFAULT_ORG,
            ],
            self::ENTITY_DEFAULT_CONFIG
        );

        return [$newPessoa, $newPessoaH];
    }

    public function setEnderecoParcDefaultOrg(
        string  $nrParcNegocio,
        string  $cdPais,
        string  $sgEstado,
        string  $cdMunicipio
    ) {
        $newEnderecoParc = $this->enderecoParcService->set(
            [
                'NRENDERECOPARC'    => NovoCodigo::geraCodigo(Util::DEFAULT_ORG, 'ENDERECOPARC', 1, 12, true),
                'NRPARCNEGOCIO'     => $nrParcNegocio,
                'CDTIPOENDERECO'    => 'PRINCIPAL',
                'CDPAIS'            => $cdPais,
                'SGESTADO'          => $sgEstado,
                'CDMUNICIPIO'       => $cdMunicipio,
                'DTINCLUSAO'        => DateUtil::getDataAtual(),
                'NRORG'             => Util::DEFAULT_ORG,
            ],
            self::ENTITY_DEFAULT_CONFIG
        );

        return $newEnderecoParc;
    }

}